;; $Id: gnus-vm.el,v 1.4 1994/06/22 06:21:25 etxbyg Exp $
;; $Source: /home/etxus1/5/etxbyg/.local/lib/emacs/RCS/gnus-vm.el,v $

;;; gnus-vm.el -- GNUS mail methods using VM
;;; Rick Sladkey <jrs@world.std.com>

;;;(setq gnus-mail-reply-method 'gnus-mail-reply-using-vm)
;;;(setq gnus-mail-other-window-method 'gnus-mail-other-window-using-vm)
;;;(setq gnus-mail-forward-method 'gnus-mail-forward-using-vm)
;;;(add-hook 'news-reply-mode-hook
;;;          '(lambda () (local-set-key "\C-c\C-y" 'gnus-yank-article)))

(defun gnus-mail-reply-using-vm (&optional yank)
  "Compose reply mail using VM.
Optional argument YANK means yank original article."
  (interactive "P")
  (gnus-mail-using-vm 'reply yank))

(defun gnus-mail-other-window-using-vm ()
  "Compose mail in the other window using VM."
  (interactive)
  (gnus-mail-using-vm 'mail))

(defun gnus-mail-forward-using-vm ()
  "Forward the current message to another person using VM."
  (interactive)
  (gnus-mail-using-vm 'forward))

(defvar gnus-vm-check-window-system nil
  "try to load win-vm if window-system.")

(defun gnus-mail-using-vm (action &optional yank)
  ;; This function treats the current buffer as an article to be
  ;; replied to or forwarded.  Action is one of reply, mail,
  ;; or forward.
  (if gnus-vm-check-window-system
      (condition-case nil
	  (require 'win-vm)
	(error nil)))
  (require 'vm)
  (let ((article (current-buffer))
	mail)
    (save-excursion
      (if (eq action 'mail)
	  (progn
	    (vm-mail)
	    (setq mail (current-buffer)))
	(save-window-excursion
	  (gnus-article-prepare gnus-current-article t)
	  (save-restriction
	    (widen)
	    (let ((tmp-folder (generate-new-buffer " *tmp-folder*"))
		  (start (point-min))
		  (end (if (eq action 'forward)
			   (point-max)
			 (goto-char (point-min))
			 (search-forward "\n\n" nil 'move)
			 (point))))
	      (set-buffer tmp-folder)
	      (insert "From \n")
	      (insert-buffer-substring article start end)
	      (goto-char (point-min))
	      (while (re-search-forward "\n\nFrom " nil t)
		(replace-match "\n\n>From "))
	      (vm-mode)
	      (if (eq action 'forward)
		  (vm-forward-message)
		(vm-reply 1))
	      (setq mail (current-buffer))
	      (kill-buffer tmp-folder))))))
    (if (eq action 'forward)
	(switch-to-buffer mail)
      (pop-to-buffer mail))
    (use-local-map (copy-keymap (current-local-map)))
    (local-set-key "\C-c\C-y" 'gnus-yank-article)
    (and yank
	 (gnus-yank-article gnus-current-article))))

(defvar gnus-included-text-prefix "    "
  "*String used to prefix included text in replies.
This variable is only used when a prefix argument is given to
`gnus-yank-article'.")

(defun gnus-yank-article (article &optional prefix)
  ;; Based on vm-yank-message by Kyle Jones.
  "Yank article number N into the current buffer at point.
When called interactively N is read from the minibuffer.

This command is meant to be used in GNUS created Mail mode buffers;
the yanked article comes from the newsgroup containing the article
you are replying to or forwarding.

All article headers are yanked along with the text.  Point is left
before the inserted text, the mark after.  Any hook functions bound to
`mail-citation-hook' are run, after inserting the text and setting
point and mark.

Prefix arg means to ignore `mail-citation-hook', don't set the mark,
prepend the value of `gnus-included-text-prefix' to every yanked line.
For backwards compatibility, if `mail-citation-hook' is set to nil,
`mail-yank-hooks' is run instead.  If that is also nil, a default
action is taken."
  (interactive
   (list
    (let ((result 0)
	  default prompt)
      (setq default (int-to-string gnus-current-article)
	    prompt (if default
		       (format "Yank article number: (default %s) " default)
		     "Yank article number: "))
      (while (zerop result)
	(setq result (read-string prompt))
	(and (string= result "") default (setq result default))
	(setq result (string-to-int result)))
      result)
    current-prefix-arg))
  (save-restriction
    (widen)
    (save-excursion
      (let ((message (current-buffer))
	    (start (point))
	    end)
	(set-buffer gnus-article-buffer)
	(gnus-article-prepare article t)
	(save-restriction
	  (widen)
	  (append-to-buffer message (point-min) (point-max)))
	(set-buffer message)
	(setq end (point))
	(goto-char start)
	(if (or prefix
		(not (or mail-citation-hook mail-yank-hooks)))
	    (save-excursion
	      (while (< (point) end)
		(insert gnus-included-text-prefix)
		(forward-line 1)))
	  (push-mark (point-max))
	  (cond
	   (mail-citation-hook (run-hooks 'mail-citation-hook))
	   (mail-yank-hooks    (run-hooks 'mail-yank-hooks))))))))

(provide 'gnus-vm)

;;; gnus-vm.el ends here
